/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef _KEY_H_
#define _KEY_H_

#if defined(_MSC_VER) || defined(__GNUC__) || defined(__clang__)
#pragma message("key.h is deprecated. Please include keyhi.h instead.")
#endif

#include "keyhi.h"

#endif /* _KEY_H_ */
